(function (global) {
    "use strict";

    global.ND = global.ND || {};
    global.ND.Portal = global.ND.Portal || {};
    global.ND.Portal.API = (function() {
        window.webapi = window.webapi || {}
		webapi.safeAjax = safeAjax;

        function safeAjax(ajaxOptions) {
			var deferredAjax = $.Deferred();
            if (shell === 'undefined') {
                return safeAjax(ajaxOptions);
            }
	
			shell.getTokenDeferred().done(function (token) {
				// add headers for AJAX
				if (!ajaxOptions.headers) {
					$.extend(ajaxOptions, {
						headers: {
							"__RequestVerificationToken": token
						}
					}); 
				} else {
					ajaxOptions.headers["__RequestVerificationToken"] = token;
				}
				$.ajax(ajaxOptions)
					.done(function(data, textStatus, jqXHR) {
						validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
					}).fail(deferredAjax.reject); //AJAX
			}).fail(function () {
				deferredAjax.rejectWith(this, arguments); // on token failure pass the token AJAX and args
			});
	
			return deferredAjax.promise();	
		}

        
        var appAjax = function (processingMsg, ajaxOptions) {
            console.log(processingMsg);
            return safeAjax(ajaxOptions)
                .fail(function (response) {
                    if (response.responseJSON) {
                        if (response.responseJSON.error.innererror) {
                            console.log(response.responseJSON.error.innererror.message);
                        } else {
							console.log(response.responseJSON.error.message);
                        }
                    } else {
                        console.log("Error Web API is not available...");
                    }
                })
                .always(ND.Portal.Notification.hide);
        };

        var asyncAppAjax = async function (ajaxOptions) {
            var deferredAjax = $.Deferred();
            shell
                .getTokenDeferred()
                .done(function (token) {
                    // Add headers for ajax
                    if (!ajaxOptions.headers) {
                        $.extend(ajaxOptions, {
                            headers: {
                                __RequestVerificationToken: token,
                            },
                        });
                    } else {
                        ajaxOptions.headers["__RequestVerificationToken"] = token;
                    }
                    $.ajax(ajaxOptions)
                        .done(function (data, textStatus, jqXHR) {
                            validateLoginSession(data, textStatus, jqXHR, deferredAjax.resolve);
                        })
                        .fail(deferredAjax.reject); //ajax
                })
                .fail(function () {
                    deferredAjax.rejectWith(this, arguments); // On token failure pass the token ajax and args
                });

            return deferredAjax.promise();
        };
        
		var handleApiRequest = function (serviceUrl, createRowCallback, postProcessingCallback) {
			appAjax('Retrieving records...', {
                type: "GET",
                url: serviceUrl,
                contentType: "application/json",
                success: function (response) {
                    var data = JSON.parse(response);
					for (let i = 0; i < data.results.length; i++) {
						createRowCallback(data.results[i]);
					}
                    if (postProcessingCallback) {
                        postProcessingCallback(data.results);
                    }
                },
                failure: function (res) {
                }
            });
            
		}

		var getMyAccounts = function(userid) {
			appAjax('Retrieving records...', {
				type: "GET",
				url: `/Services?name=ND Service: Get My Companies`,
				contentType: "application/json",
				success: function (response) {
					var data = JSON.parse(response);
					if (data.results.length > 0) {
						data.results.forEach((element) => {
							_generateAccountDetailList(element, "my-company-list");
						});
					} else {
						$("#no-companies-alert").removeClass("hidden");
					}
				},
				failure: function (res) {
					currentPage = previousPage;
				}
			});
		}

		var _generateAccountDetailList = function(element, detailListName) {
			$(`#${detailListName}`).append(`
                <div>
                    <h2 class="h4 page-header">${element.Name}</h2>
                    <dl class="data-list data-list-label-width-md">
                        <dt>Phone</dt>
                        <dd>${element.Telephone ?? '&ndash;'}</dd>

                        <dt>Email</dt>
                        <dd>${element.EmailAddress ?? '&ndash;'}</dd>

                        <dt>Address</dt>
                        <dd>${ND.Portal.Utilities.createCompositeAddress(element.line1, element.line2, element.city, element.state, element.zip, element.country)}</dd>
                    </dl>
                </div>
            `);
		}

        var updateContactNotificationPreferences = (contactId, value, onSuccess, onError) => {
            _updateAPIRequest(
                `contacts`,
                contactId,
                { "donotbulkemail": value },
                onSuccess, onError
            );
        }

        var upsertContactUpdateRequest = (user, id, updates, onSuccess, onError) => {
            if (id) {
                _updateAPIRequest('ndland_contactupdaterequests', id, updates, onSuccess, onError);
            }
            else {
                updates["ndland_ContactId@odata.bind"] = `/contacts(${user})`;
                _createApiRequest('ndland_contactupdaterequests', updates, onSuccess, onError);
            }
        }

        var createMessageAndUploadFiles = (values, files, onSuccess, onError) => {
            _createApiRequest('ndland_portalmessages', values, 
                (id) => { 
                    var status = {statuscode: 769150001, statecode: 0};
                    _handleMessageCenterFileSave(id, onSuccess, onError);
                }, 
                onError);
        }

        var markMessageAsRead = (id, onSuccess, onError) => {
            var status = {statuscode: 769150002};
            _updateAPIRequest('ndland_portalmessages', id, status, onSuccess, onError);
        }


        var _handleMessageCenterFileSave = async (id, onSuccess, onError) => {
            var isSuccess = false;
            for (let i = 0; i < uploadedFiles.length; i++) {
                const file = uploadedFiles[i];
                const fileName = file.name;
                const mimeType = file.type;
                const fileData = await ND.Portal.Documents.storeFileData([file]);

                const base64Content = fileData[0].base64Content;
                const note = {
                    "objectid_ndland_portalmessage@odata.bind": `/ndland_portalmessages(${id})`,
                    "objecttypecode": "ndland_portalmessage",
                    "subject": fileName,
                    "filename": fileName,
                    "documentbody": base64Content,
                    "mimetype": mimeType
                };

                await ND.Portal.API.appAjax('Saving record...', {
                    type: "POST",
                    url: "/_api/annotations",
                    contentType: "application/json",
                    data: JSON.stringify(note),
                    success: function() {
                    },
                    error: function(xhr, status, error){
                        if (onError) onError(_createErrorResponse(xhr, status, error)) 
                    }
                });
            }
            if (onSuccess) {
                onSuccess(id);
            }
        };

        var _updateAPIRequest = (entity, id, values, onSuccess, onError) => {
            webapi.safeAjax({
                type: "PATCH",
                url: `/_api/${entity}(${id})`,
                contentType:"application/json",
                data: JSON.stringify(values),
                success: function() {
                    if (onSuccess) onSuccess(id);
                },
                // On error
                error: function(xhr, status, error){
                    if (onError) onError(_createErrorResponse(xhr, status, error)) 
                }
            })
        }

        var _createApiRequest = (entity, values, onSuccess, onError) => {
            webapi.safeAjax({
                type: "POST",
                url: `/_api/${entity}`,
                contentType:"application/json",
                data: JSON.stringify(values),
                success: function (res, status, xhr) {
                    if (onSuccess) onSuccess(_createSuccessResponse(xhr.getResponseHeader("entityid"))) 
                },
                // On error
                error: function(xhr, status, error){
                    if (onError) onError(_createErrorResponse(xhr, status, error)) 
                }
            })
        }

        var _createSuccessResponse = (results) => {
            return results;
        }

        var _createErrorResponse = (xhr, status, error) => {
            var errorMessage = xhr.status + ': ' + xhr.statusText;
            console.log(errorMessage);
            return errorMessage;
        }
		
		var writeIntersectSubgrid = (totalRecordCount, subgridTableName, intersectLogicalName, entityLogicalName, currentItem, removeTractList, createRowCallback, isRelated) => {
			let inputObj = {
						"intersects": [
							{
								"intersectLogicalName": intersectLogicalName,
								"records" : []
							}
						]
			};
			const tableMappings = {
				"ndland_applicationleaseagreement": { nameKey: "LeaseAgreementName", idKey: "LeaseAgreementId" },
				"ndland_applicationleaseagreementcount": { nameKey: "LeaseAgreementName", idKey: "LeaseAgreementId" },
				"ndland_applicationleaseagreementownership": { nameKey: "LeaseAgreementName", idKey: "Id" },
				"ndland_applicationleaseagreementownershipcount": { nameKey: "LeaseAgreementName", idKey: "Id" },
				"ndland_applicationtractcount": { nameKey: "TractName", idKey: "TractId" },
				"ndland_applicationtract": { nameKey: "TractName", idKey: "TractId" },
                "ndland_applicationlandunit": { nameKey: "LandUnitName", idKey: "LandUnitId" },
				"ndland_applicationlandunitcount": { nameKey: "LandUnitName", idKey: "LandUnitId" },
				"ndland_applicationaggregatetype": { nameKey: "AggregateTypeName", idKey: "AggregateTypeId" },
				"ndland_applicationassignee": { nameKey: "AssigneeName", idKey: "AssigneeId" },
                "ndland_applicationassigneecount": { nameKey: "AssigneeName", idKey: "AssigneeId" },
                "ndland_applicationwell": { nameKey: "WellName", idKey: "WellId" },
				"ndland_applicationwellcount": { nameKey: "WellName", idKey: "WellId" },
				"ndland_applicationweedpest": { nameKey: "ProductName", idKey: "ProductId" },
                "ndland_applicationweedpestcount": { nameKey: "ProductName", idKey: "ProductId" }
			};
			let tableKeys = tableMappings[intersectLogicalName];
			
			if (!tableKeys) return;
			
            var surfaceOwnership;
            var name;
            var id;
            if (typeof currentItem === 'object' && currentItem !== null && 'name' in currentItem && 'id' in currentItem) {
                name = currentItem.name;
                id = currentItem.id;
            } else {
                var currentTd = currentItem.closest('td');
                var nextTd = currentTd.next('td');
                surfaceOwnership = nextTd.text();
                name = currentItem.val() || currentItem.text();
                id = currentItem.attr("id");
            }

            let removedTractIndex = removeTractList.indexOf(id);
            if (removedTractIndex !== -1) {
                removeTractList.splice(removedTractIndex, 1);  // Remove the tract from the track remove list as it was re-added by the user
            }

			let result = {
				[tableKeys.nameKey]: name,
				[tableKeys.idKey]: id,
				TotalRecordCount: totalRecordCount
			};

            let intersect = {
                "entityLogicalName": entityLogicalName,
                "id": id,
                "name": name,
                "additionalValues": null
            };
			if (isRelated) {
				intersect["isRelated"] = isRelated; //related records i.e. related tracts
			}
			if (intersectLogicalName == "ndland_applicationleaseagreement" || intersectLogicalName == "ndland_applicationleaseagreementcount") {
				result.ExpirationDate = currentItem.parent().parent().parent().attr("data-expirationdate");
				result.PrimaryLessee = currentItem.parent().parent().parent().attr("data-primarylessee");
				result.AnnualRent = currentItem.parent().parent().parent().attr("data-annualrent");
				result.LeasedAcres = currentItem.parent().parent().parent().attr("data-leasedacres");
				result.PercentOwnership = currentItem.parent().parent().parent().attr("data-owernship");
				result.Depth = currentItem.parent().parent().parent().attr("data-depthid");
				
				intersect.ExpirationDate = currentItem.parent().parent().parent().attr("data-expirationdate");
				intersect.PrimaryLessee = currentItem.parent().parent().parent().attr("data-primarylessee");
				intersect.AnnualRent = currentItem.parent().parent().parent().attr("data-annualrent");
				intersect.LeasedAcres = currentItem.parent().parent().parent().attr("data-leasedacres");
				intersect.PercentOwnership = currentItem.parent().parent().parent().attr("data-owernship");
				intersect.Depth = currentItem.parent().parent().parent().attr("data-depthid");
			}
            
			if (intersectLogicalName == "ndland_applicationleaseagreementownership" || intersectLogicalName == "ndland_applicationleaseagreementownershipcount") {
                result.LeaseAgreementId = currentItem.parent().parent().parent().attr("data-leaseagreementid");
                result.LeaseNumber = currentItem.parent().parent().parent().attr("data-leasenum");
                result.Assignee = currentItem.parent().parent().parent().attr("data-assignee");
				result.PercentOwnership = currentItem.parent().parent().parent().attr("data-owernship");
				result.Stratigraphic = currentItem.parent().parent().parent().attr("data-stratigraphic");

                intersect.LeaseAgreementId = currentItem.parent().parent().parent().attr("data-leaseagreementid");
				intersect.LeaseNumber = currentItem.parent().parent().parent().attr("data-leasenum");
                intersect.Assignee = currentItem.parent().parent().parent().attr("data-assignee");
				intersect.PercentOwnership = currentItem.parent().parent().parent().attr("data-owernship");
				intersect.Stratigraphic = currentItem.parent().parent().parent().attr("data-stratigraphic");
			}

            if (!('name' in currentItem && 'id' in currentItem) && (intersectLogicalName == "ndland_applicationtract" || intersectLogicalName == "ndland_applicationtractcount")) {
				result.LeaseNum = currentItem.parent().parent().parent().parent().find("td[data-leasenum]").attr("data-leasenum");
				intersect.LeaseNum = currentItem.parent().parent().parent().parent().find("td[data-leasenum]").attr("data-leasenum");
                inputObj.LeaseNum = intersect.LeaseNum;
            }

            if (intersectLogicalName == "ndland_applicationwell" || intersectLogicalName == "ndland_applicationwellcount") {
				result.APINumber = currentItem.parent().parent().parent().attr("data-apinumber");
				result.NDICFileNumber = currentItem.parent().parent().parent().attr("data-ndicfilenumber");
				
				intersect.APINumber = currentItem.parent().parent().parent().attr("data-apinumber");
				intersect.NDICFileNumber = currentItem.parent().parent().parent().attr("data-ndicfilenumber");
			}
			
            var portalData = $("#ndland_portaldata").val();
            if (portalData.length > 0 && portalData !== "{}") {
                inputObj = JSON.parse($("#ndland_portaldata").val());
                if (!inputObj.intersects) {
                    inputObj.intersects = [];
                }
            }
            let rootIndex = inputObj.intersects.findIndex(x => x.intersectLogicalName == intersectLogicalName);
            if (rootIndex == -1) {
                var rootObj = {
                    "intersectLogicalName" : intersectLogicalName,
                    "records" : [intersect]
                }
                inputObj.intersects.push(rootObj);
                if (createRowCallback) {
                    $(`#${subgridTableName} .empty-message`).hide()
                    var rowHtml = createRowCallback(result, true);
                    $(`#${subgridTableName}`).find("tbody").append(rowHtml);
                }

            } else {
                let childIndex = inputObj.intersects[rootIndex].records.findIndex(x => x.id == id);
                if (childIndex == -1) {//avoid adding duplicates
                    inputObj.intersects[rootIndex].records.push(intersect);
                    if (createRowCallback) {
                        $(`#${subgridTableName} .empty-message`).hide()
                        var rowHtml = createRowCallback(result, true);
                        $(`#${subgridTableName}`).find("tbody").append(rowHtml);
                    }
                }
            }
			
			return inputObj;
		}
		
		var writeIntersectDropdown = (totalRecordCount, subgridTableName, intersectLogicalName, entityLogicalName, currentId) => {
			let intersectsObj = 
            [
                {
                    "intersectLogicalName": intersectLogicalName,
                    "records" : []
                }
            ];
			const tableMappings = {
				"ndland_applicationleaseagreement": { nameKey: "LeaseAgreementName", idKey: "LeaseAgreementId" },
				"ndland_applicationtract": { nameKey: "TractName", idKey: "TractId" },
				"ndland_applicationaggregatetype": { nameKey: "AggregateTypeName", idKey: "AggregateTypeId" },
				"ndland_applicationassignee": { nameKey: "AssigneeName", idKey: "AssigneeId" },
				"ndland_applicationweedpesttype": { nameKey: "WeedPestTypeName", idKey: "WeedPestTypeId" }
			};
			let tableKeys = tableMappings[intersectLogicalName];
			
			if (!tableKeys) return;
			
            var name = $(`#${currentId}`).val();
            var id = currentId;

			let result = {
				[tableKeys.nameKey]: name,
				[tableKeys.idKey]: id,
				TotalRecordCount: totalRecordCount
			};

            let intersect = {
                "entityLogicalName": entityLogicalName,
                "id": id,
                "name": name,
                "additionalValues": null
            };
			
            var portalData =  ND.Portal.Utilities.getPortalData();
            let intersects = ND.Portal.Utilities.getPortalDataProperty('intersects', intersectsObj);
            let rootIndex = intersectsObj.findIndex(x => x.intersectLogicalName == intersectLogicalName);
            if (rootIndex == -1) {
                var rootObj = {
                    "intersectLogicalName" : intersectLogicalName,
                    "records" : [intersect]
                }
                intersectsObj.push(rootObj);
				$(`#${id}`).prop('selected', true);

            } else {
				
				let childIndex = intersectsObj[rootIndex].records.findIndex(x => x.id == id);
                if (childIndex == -1) {//avoid adding duplicates
                    intersectsObj[rootIndex].records.push(intersect);
					$(`#${id}`).prop('selected', true);
                }
            }
			
            portalData.intersects = intersectsObj
			return portalData;
		}

        return {
            appAjax: appAjax,
            asyncAppAjax: asyncAppAjax,
            handleApiRequest: handleApiRequest,
            getMyAccounts: getMyAccounts,
            updateContactNotificationPreferences: updateContactNotificationPreferences,
            upsertContactUpdateRequest: upsertContactUpdateRequest,
            createMessageAndUploadFiles: createMessageAndUploadFiles,
            writeIntersectDropdown: writeIntersectDropdown,
            writeIntersectSubgrid: writeIntersectSubgrid,
            markMessageAsRead: markMessageAsRead
        }
    })();

})(this);